from pydantic import BaseModel, Field
from typing import List, Optional

from chiefpay.types.enums import InvoiceStatus
from chiefpay.types.transaction import LastTransaction


class Address(BaseModel):
    chain: str
    token: str
    method_name: str = Field(alias='methodName')
    address: str
    token_rate: Optional[str] = Field(alias='tokenRate', default=None)


class FiatDetails(BaseModel):
    name: str
    amount: str
    payed_amount: str = Field(alias="payedAmount")
    fee_rate: str = Field(alias="feeRate")
    bank: str
    requisites: str
    card_owner: str = Field(alias="cardOwner")


class Invoice(BaseModel):
    id: str
    order_id: str = Field(alias="orderId")
    payed_amount: str = Field(alias="payedAmount")
    fee_included: bool = Field(alias="feeIncluded")
    accuracy: str
    fee_rate: str = Field(alias="feeRate")
    created_at: str = Field(alias="createdAt")
    expired_at: str = Field(alias="expiredAt")
    status: InvoiceStatus
    addresses: List[Address]
    description: Optional[str] = Field(default=None)
    amount: Optional[str] = Field(default="0")
    fiat_details: Optional[List[FiatDetails]] = Field(alias="FiatDetails", default=None)
    last_transaction: Optional[LastTransaction] = Field(alias="lastTransaction", default=None)
    url: str
    url_success: Optional[str] = Field(alias="urlSuccess", default=None)
    url_return: Optional[str] = Field(alias="urlReturn", default=None)

    class Config:
        populate_by_name = True



from enum import Enum
from typing import List, Optional


class ChiefPayErrorCode(Enum):
    INVALID_ARGUMENT = "INVALID_ARGUMENT"
    ALREADY_EXISTS = "ALREADY_EXISTS"
    NOT_FOUND = "NOT_FOUND"
    INTERNAL = "INTERNAL"
    OUT_OF_RANGE = "OUT_OF_RANGE"
    UNAUTHENTICATED = "UNAUTHENTICATED"
    PERMISSION_DENIED = "PERMISSION_DENIED"


class ChiefPayError(Exception):
    def __init__(self, message: str):
        self.message = message
        super().__init__(message)


class APIError(ChiefPayError):
    def __init__(
        self,
        message: str,
        code: Optional[str] = None,
        fields: Optional[List[str]] = None,
        status_code: Optional[int] = None,
    ):
        self.code = (
            ChiefPayErrorCode(code) if code in ChiefPayErrorCode.__members__ else None
        )
        self.fields = fields or []
        self.status_code = status_code
        super().__init__(f"{message} (Code: {code}, Fields: {fields})")


class ManyRequestsError(ChiefPayError):
    def __init__(self):
        super().__init__("Too Many Requests", code="OUT_OF_RANGE")


class InvalidJSONError(ChiefPayError):
    def __init__(self, message: str = "Invalid JSON response"):
        super().__init__(message)


class TransportError(ChiefPayError):
    def __init__(self, status_code: int, message: str):
        self.status_code = status_code
        super().__init__(f"Transport Error: {status_code}: {message}")


class SocketError(ChiefPayError):
    def __init__(self, message: str):
        super().__init__(message)



__all__ = (
    'Invoice',
    'Address',
    'FiatDetails',
    'Wallet',
    'Rate',
    'Transaction',
    'NotificationTransaction',
    'NotificationInvoice',
    'TransactionsHistory',
    'InvoicesHistory',
    'InvoiceStatus',
)

from chiefpay.types.invoice import Invoice, Address, FiatDetails
from chiefpay.types.history import TransactionsHistory, InvoicesHistory
from chiefpay.types.wallet import Wallet
from chiefpay.types.rates import Rate
from chiefpay.types.transaction import Transaction
from chiefpay.types.notification import NotificationTransaction, NotificationInvoice
from chiefpay.types.enums import InvoiceStatus